# Copyright © Michal Čihař <michal@weblate.org>
#
# SPDX-License-Identifier: MIT
"""
Type stubs for unicode-segmentation-rs

This module provides Unicode text segmentation and width calculation.
"""

def graphemes(text: str, is_extended: bool) -> list[str]: ...
def grapheme_indices(text: str, is_extended: bool) -> list[tuple[int, str]]: ...
def split_word_bounds(text: str) -> list[str]: ...
def split_word_bound_indices(text: str) -> list[tuple[int, str]]: ...
def unicode_words(text: str) -> list[str]: ...
def unicode_sentences(text: str) -> list[str]: ...
def text_width(text: str) -> int: ...
def gettext_wrap(text: str, width: int) -> list[str]: ...
